@@ -3,6 +3,7 @@
 package node
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"net"
@@ -13,13 +14,11 @@ import (
 	"github.com/vishvananda/netlink"
 	"k8s.io/klog"
 
-	metrics "github.com/openshift/sdn/pkg/network/node/metrics"
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/types"
-	kubeutilnet "k8s.io/apimachinery/pkg/util/net"
 	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/strategicpatch"
 	kwait "k8s.io/apimachinery/pkg/util/wait"
@@ -34,6 +33,7 @@ import (
 	"k8s.io/kubernetes/pkg/apis/core/v1/helper"
 	ktypes "k8s.io/kubernetes/pkg/kubelet/types"
 	kubeproxyconfig "k8s.io/kubernetes/pkg/proxy/apis/config"
+	"k8s.io/kubernetes/pkg/util/iptables"
 	taints "k8s.io/kubernetes/pkg/util/taints"
 	kexec "k8s.io/utils/exec"
 
@@ -64,8 +64,8 @@ type osdnPolicy interface {
 }
 
 type OsdnNodeConfig struct {
-	Hostname string
-	SelfIP   string
+	NodeName string
+	NodeIP   string
 
 	NetworkClient networkclient.Interface
 	KClient       kubernetes.Interface
@@ -74,27 +74,28 @@ type OsdnNodeConfig struct {
 	KubeInformers    informers.SharedInformerFactory
 	NetworkInformers networkinformers.SharedInformerFactory
 
-	IPTablesSyncPeriod time.Duration
-	ProxyMode          kubeproxyconfig.ProxyMode
-	MasqueradeBit      *int32
+	IPTables      iptables.Interface
+	ProxyMode     kubeproxyconfig.ProxyMode
+	MasqueradeBit *int32
 }
 
 type OsdnNode struct {
-	policy             osdnPolicy
-	kClient            kubernetes.Interface
-	networkClient      networkclient.Interface
-	recorder           record.EventRecorder
-	oc                 *ovsController
-	networkInfo        *common.ParsedClusterNetwork
-	podManager         *podManager
-	clusterCIDRs       []string
-	localSubnetCIDR    string
-	localGatewayCIDR   string
-	localIP            string
-	hostName           string
-	useConnTrack       bool
-	iptablesSyncPeriod time.Duration
-	masqueradeBit      uint32
+	policy           osdnPolicy
+	kClient          kubernetes.Interface
+	networkClient    networkclient.Interface
+	recorder         record.EventRecorder
+	oc               *ovsController
+	networkInfo      *common.ParsedClusterNetwork
+	podManager       *podManager
+	ipt              iptables.Interface
+	nodeIPTables     *NodeIPTables
+	clusterCIDRs     []string
+	localSubnetCIDR  string
+	localGatewayCIDR string
+	localIP          string
+	hostName         string
+	useConnTrack     bool
+	masqueradeBit    uint32
 
 	// Synchronizes operations on egressPolicies
 	egressPoliciesLock sync.Mutex
@@ -117,7 +118,7 @@ func New(c *OsdnNodeConfig) (*OsdnNode, error) {
 		return nil, fmt.Errorf("could not get ClusterNetwork resource: %v", err)
 	}
 
-	if err := c.setNodeIP(networkInfo); err != nil {
+	if err := c.validateNodeIP(networkInfo); err != nil {
 		return nil, err
 	}
 
@@ -145,13 +146,13 @@ func New(c *OsdnNodeConfig) (*OsdnNode, error) {
 		return nil, fmt.Errorf("%q plugin is not compatible with proxy-mode %q", networkInfo.PluginName, c.ProxyMode)
 	}
 
-	klog.Infof("Initializing SDN node of type %q with configured hostname %q (IP %q)", networkInfo.PluginName, c.Hostname, c.SelfIP)
+	klog.Infof("Initializing SDN node %q (%s) of type %q", c.NodeName, c.NodeIP, networkInfo.PluginName)
 
 	ovsif, err := ovs.New(kexec.New(), Br0, minOvsVersion)
 	if err != nil {
 		return nil, err
 	}
-	oc := NewOVSController(ovsif, pluginId, useConnTrack, c.SelfIP)
+	oc := NewOVSController(ovsif, pluginId, useConnTrack, c.NodeIP)
 
 	masqBit := uint32(0)
 	if c.MasqueradeBit != nil {
@@ -164,57 +165,34 @@ func New(c *OsdnNodeConfig) (*OsdnNode, error) {
 	}
 
 	plugin := &OsdnNode{
-		policy:             policy,
-		kClient:            c.KClient,
-		networkClient:      c.NetworkClient,
-		recorder:           c.Recorder,
-		oc:                 oc,
-		networkInfo:        networkInfo,
-		podManager:         newPodManager(c.KClient, policy, networkInfo.MTU, oc),
-		localIP:            c.SelfIP,
-		hostName:           c.Hostname,
-		useConnTrack:       useConnTrack,
-		iptablesSyncPeriod: c.IPTablesSyncPeriod,
-		masqueradeBit:      masqBit,
-		egressPolicies:     make(map[uint32][]networkapi.EgressNetworkPolicy),
-		egressDNS:          egressDNS,
-		kubeInformers:      c.KubeInformers,
-		networkInformers:   c.NetworkInformers,
-		egressIP:           newEgressIPWatcher(oc, c.SelfIP, c.MasqueradeBit),
+		policy:           policy,
+		kClient:          c.KClient,
+		networkClient:    c.NetworkClient,
+		recorder:         c.Recorder,
+		oc:               oc,
+		networkInfo:      networkInfo,
+		podManager:       newPodManager(c.KClient, policy, networkInfo.MTU, oc),
+		localIP:          c.NodeIP,
+		hostName:         c.NodeName,
+		useConnTrack:     useConnTrack,
+		ipt:              c.IPTables,
+		masqueradeBit:    masqBit,
+		egressPolicies:   make(map[uint32][]networkapi.EgressNetworkPolicy),
+		egressDNS:        egressDNS,
+		kubeInformers:    c.KubeInformers,
+		networkInformers: c.NetworkInformers,
+		egressIP:         newEgressIPWatcher(oc, c.NodeIP, c.MasqueradeBit),
 	}
 
-	metrics.RegisterMetrics()
+	RegisterMetrics()
 
 	return plugin, nil
 }
 
-// Set node IP if required
-func (c *OsdnNodeConfig) setNodeIP(networkInfo *common.ParsedClusterNetwork) error {
-	if len(c.Hostname) == 0 {
-		output, err := kexec.New().Command("uname", "-n").CombinedOutput()
-		if err != nil {
-			return err
-		}
-		c.Hostname = strings.TrimSpace(string(output))
-	}
-
-	if len(c.SelfIP) == 0 {
-		var err error
-		c.SelfIP, err = GetNodeIP(c.Hostname)
-		if err != nil {
-			klog.Infof("Failed to determine node address from hostname %s; using default interface (%v)", c.Hostname, err)
-			var defaultIP net.IP
-			defaultIP, err = kubeutilnet.ChooseHostInterface()
-			if err != nil {
-				return err
-			}
-			c.SelfIP = defaultIP.String()
-		}
-	}
-
-	if _, _, err := GetLinkDetails(c.SelfIP); err != nil {
+func (c *OsdnNodeConfig) validateNodeIP(networkInfo *common.ParsedClusterNetwork) error {
+	if _, _, err := GetLinkDetails(c.NodeIP); err != nil {
 		if err == ErrorNetworkInterfaceNotFound {
-			err = fmt.Errorf("node IP %q is not a local/private address (hostname %q)", c.SelfIP, c.Hostname)
+			err = fmt.Errorf("node IP %q is not a local/private address (hostname %q)", c.NodeIP, c.NodeName)
 		}
 		utilruntime.HandleError(fmt.Errorf("Unable to find network interface for node IP; some features will not work! (%v)", err))
 	}
@@ -231,33 +209,6 @@ func (c *OsdnNodeConfig) setNodeIP(networkInfo *common.ParsedClusterNetwork) err
 	return nil
 }
 
-func GetNodeIP(nodeName string) (string, error) {
-	ip := net.ParseIP(nodeName)
-	if ip == nil {
-		addrs, err := net.LookupIP(nodeName)
-		if err != nil {
-			return "", fmt.Errorf("Failed to lookup IP address for node %s: %v", nodeName, err)
-		}
-		for _, addr := range addrs {
-			// Skip loopback and non IPv4 addrs
-			if addr.IsLoopback() || addr.To4() == nil {
-				klog.V(5).Infof("Skipping loopback/non-IPv4 addr: %q for node %s", addr.String(), nodeName)
-				continue
-			}
-			ip = addr
-			break
-		}
-	} else if ip.IsLoopback() || ip.To4() == nil {
-		klog.V(5).Infof("Skipping loopback/non-IPv4 addr: %q for node %s", ip.String(), nodeName)
-		ip = nil
-	}
-
-	if ip == nil || len(ip.String()) == 0 {
-		return "", fmt.Errorf("Failed to obtain IP address from node name: %s", nodeName)
-	}
-	return ip.String(), nil
-}
-
 var (
 	ErrorNetworkInterfaceNotFound = fmt.Errorf("could not find network interface")
 )
@@ -326,14 +277,14 @@ func (node *OsdnNode) validateMTU() error {
 	needsTaint := mtu < int(node.networkInfo.MTU)+50
 	const MTUTaintKey string = "network.openshift.io/mtu-too-small"
 	mtuTooSmallTaint := &corev1.Taint{Key: MTUTaintKey, Value: "value", Effect: "NoSchedule"}
-	nodeObj, err := node.kClient.CoreV1().Nodes().Get(node.hostName, metav1.GetOptions{})
+	nodeObj, err := node.kClient.CoreV1().Nodes().Get(context.TODO(), node.hostName, metav1.GetOptions{})
 	if err != nil {
 		return fmt.Errorf("could not get Kubernetes Node object by hostname: %v", err)
 	}
 	tainted := taints.TaintExists(nodeObj.Spec.Taints, mtuTooSmallTaint)
 	if needsTaint != tainted {
 		resultErr := retry.RetryOnConflict(retry.DefaultBackoff, func() error {
-			nodeObj, err = node.kClient.CoreV1().Nodes().Get(node.hostName, metav1.GetOptions{})
+			nodeObj, err = node.kClient.CoreV1().Nodes().Get(context.TODO(), node.hostName, metav1.GetOptions{})
 			if err != nil {
 				return fmt.Errorf("could not get Kubernetes Node object by hostname: %v", err)
 			}
@@ -369,7 +320,7 @@ func (node *OsdnNode) validateMTU() error {
 				return fmt.Errorf("could not create patch for object: %v", err)
 			}
 
-			_, err = node.kClient.CoreV1().Nodes().Patch(node.hostName, types.StrategicMergePatchType, patchBytes)
+			_, err = node.kClient.CoreV1().Nodes().Patch(context.TODO(), node.hostName, types.StrategicMergePatchType, patchBytes, metav1.PatchOptions{})
 
 			return err
 		})
@@ -394,9 +345,9 @@ func (node *OsdnNode) Start() error {
 	for _, cn := range node.networkInfo.ClusterNetworks {
 		node.clusterCIDRs = append(node.clusterCIDRs, cn.ClusterCIDR.String())
 	}
-	nodeIPTables := newNodeIPTables(node.clusterCIDRs, node.iptablesSyncPeriod, !node.useConnTrack, node.networkInfo.VXLANPort, node.masqueradeBit)
 
-	if err = nodeIPTables.Setup(); err != nil {
+	node.nodeIPTables = newNodeIPTables(node.ipt, node.clusterCIDRs, !node.useConnTrack, node.networkInfo.VXLANPort, node.masqueradeBit)
+	if err = node.nodeIPTables.Setup(); err != nil {
 		return fmt.Errorf("failed to set up iptables: %v", err)
 	}
 
@@ -415,7 +366,7 @@ func (node *OsdnNode) Start() error {
 		if err := node.SetupEgressNetworkPolicy(); err != nil {
 			return err
 		}
-		if err := node.egressIP.Start(node.networkInformers, nodeIPTables); err != nil {
+		if err := node.egressIP.Start(node.networkInformers, node.nodeIPTables); err != nil {
 			return err
 		}
 	}
@@ -455,8 +406,7 @@ func (node *OsdnNode) Start() error {
 
 	go kwait.Forever(node.policy.SyncVNIDRules, time.Hour)
 	go kwait.Forever(func() {
-		metrics.GatherPeriodicMetrics()
-		node.oc.ovs.UpdateOVSMetrics()
+		gatherPeriodicMetrics(node.oc.ovs)
 	}, time.Minute*2)
 
 	return nil
@@ -546,7 +496,7 @@ func (node *OsdnNode) GetRunningPods(namespace string) ([]corev1.Pod, error) {
 		LabelSelector: labels.Everything().String(),
 		FieldSelector: fieldSelector.String(),
 	}
-	podList, err := node.kClient.CoreV1().Pods(namespace).List(opts)
+	podList, err := node.kClient.CoreV1().Pods(namespace).List(context.TODO(), opts)
 	if err != nil {
 		return nil, err
 	}
@@ -615,3 +565,7 @@ func (node *OsdnNode) handleDeleteService(obj interface{}) {
 	klog.V(5).Infof("Watch %s event for Service %q", watch.Deleted, serv.Name)
 	node.DeleteServiceRules(serv)
 }
+
+func (node *OsdnNode) ReloadIPTables() error {
+	return node.nodeIPTables.syncIPTableRules()
+}
